@@ -27,9 +27,8 @@ class Program():
     """
 
     def __init__(self, source, name=None, language='C++', 
-                       workdir=Path(__file__).parent / 'programs', 
-                       force_build=False):
-        self.language = language_[language]
+                       workdir=Path(__file__).parent / 'programs'):
+        self.language = language_(language)
 
         self.workdir = workdir
         self.name = name or str(uuid4())
@@ -37,7 +36,8 @@ class Program():
         self.language.write_source(self.workdir, self.name, source)
         self.stdout, self.stderr = self.language.build(self.workdir, self.name)
 
-        assert force_build or not self.stderr, self.stderr
+        # Please send your opinions on 'not not' vs 'bool()' to dont@vadim.me
+        self.compile_error = not not self.stderr
 
     def __lt__(self, other):
         return self.name < other.name
@@ -58,7 +58,17 @@ class Program():
         assert force or not self.stderr, self.stderr
         return self.stdout.splitlines()
 
-    def score(self, test_cases, force=True):
+    def score(self, test_cases, force=True, cache=True):
+        """
+        Test the program against a list of input output pairs
+
+        If 'force=True', program outputs will be checked even if there are errors on stderr
+        If 'cache=True', all the test logs will be stored in `program.test_runs` attribute
+        """
+
+        if self.compile_error:
+            return 0
+
         self.test_runs = []
         for input_lines, expected_output_lines in test_cases:
             try:
@@ -69,7 +79,12 @@ class Program():
                 test_run = TestRun(input_lines, expected_output_lines, [], 0)
 
             self.test_runs.append(test_run)
-        return mean([run.correctness for run in self.test_runs])
+        self.score = mean([run.correctness for run in self.test_runs])
+
+        if not cache:
+            del self.test_runs
+
+        return self.score
 
     def save(self, path):
         self.language.copy_source(self.workdir, self.name, path)
